# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_igroup \
    import NetAppOntapIgroup as igroup  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.kind = kind
        self.data = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'igroup':
            xml = self.build_igroup()
        if self.kind == 'igroup_no_initiators':
            xml = self.build_igroup_no_initiators()
        self.xml_out = xml
        return xml

    @staticmethod
    def build_igroup():
        ''' build xml data for initiator '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'initiator-group-info': {
                    'initiators': [
                        {
                            'initiator-info': {
                                'initiator-name': 'init1'
                            }},
                        {
                            'initiator-info': {
                                'initiator-name': 'init2'
                            }}
                    ]
                }
            }
        }
        xml.translate_struct(attributes)
        return xml

    @staticmethod
    def build_igroup_no_initiators():
        ''' build xml data for igroup with no initiators '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'initiator-group-info': {
                    'vserver': 'test'
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()

    def mock_args(self):
        return {
            'vserver': 'vserver',
            'name': 'test',
            'initiators': 'init1',
            'ostype': 'linux',
            'initiator_group_type': 'fcp',
            'bind_portset': 'true',
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password'
        }

    def get_igroup_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_igroup object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_igroup object
        """
        obj = igroup()
        obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            obj.server = MockONTAPConnection()
        else:
            obj.server = MockONTAPConnection(kind=kind)
        return obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            igroup()

    def test_get_nonexistent_igroup(self):
        ''' Test if get_igroup returns None for non-existent igroup '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_igroup_mock_object().get_igroup('dummy')
        assert result is None

    def test_get_existing_igroup_with_initiators(self):
        ''' Test if get_igroup returns list of existing initiators '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_igroup_mock_object('igroup').get_igroup(data['name'])
        assert data['initiators'] in result['initiators']
        assert result['initiators'] == ['init1', 'init2']

    def test_get_existing_igroup_without_initiators(self):
        ''' Test if get_igroup returns empty list() '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_igroup_mock_object('igroup_no_initiators').get_igroup(data['name'])
        assert result['initiators'] == []

    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.add_initiators')
    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.remove_initiators')
    def test_modify_initiator_calls_add_and_remove(self, remove, add):
        '''Test remove_initiator() is called followed by add_initiator() on modify operation'''
        data = self.mock_args()
        data['initiators'] = 'replacewithme'
        set_module_args(data)
        obj = self.get_igroup_mock_object('igroup')
        with pytest.raises(AnsibleExitJson) as exc:
            current = obj.get_igroup(data['name'])
            obj.apply()
        remove.assert_called_with(current['initiators'])
        add.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.modify_initiator')
    def test_modify_called_from_add(self, modify):
        '''Test remove_initiator() and add_initiator() calls modify'''
        data = self.mock_args()
        data['initiators'] = 'replacewithme'
        add, remove = 'igroup-add', 'igroup-remove'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object('igroup_no_initiators').apply()
        modify.assert_called_with('replacewithme', add)
        assert modify.call_count == 1      # remove nothing, add 1 new

    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.modify_initiator')
    def test_modify_called_from_remove(self, modify):
        '''Test remove_initiator() and add_initiator() calls modify'''
        data = self.mock_args()
        data['initiators'] = ''
        remove = 'igroup-remove'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object('igroup').apply()
        modify.assert_called_with('init2', remove)
        assert modify.call_count == 2  # remove existing 2, add nothing

    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.add_initiators')
    def test_successful_create(self, add):
        ''' Test successful create '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object().apply()
        assert exc.value.args[0]['changed']
        add.assert_called_with()

    def test_successful_delete(self):
        ''' Test successful delete '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object('igroup').apply()
        assert exc.value.args[0]['changed']

    def test_successful_modify(self):
        ''' Test successful modify '''
        data = self.mock_args()
        data['initiators'] = 'new'
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object('igroup').apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_igroup.NetAppOntapIgroup.get_igroup')
    def test_successful_rename(self, get_vserver):
        '''Test successful rename'''
        data = self.mock_args()
        data['from_name'] = 'test'
        data['name'] = 'test_new'
        set_module_args(data)
        current = {
            'initiators': ['init1', 'init2']
        }
        get_vserver.side_effect = [
            None,
            current
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_igroup_mock_object().apply()
        assert exc.value.args[0]['changed']



# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import

from ansible.modules.source_control.gitlab_hook import GitLabHook

from .gitlab import (GitlabModuleTestCase,
                     python_version_match_requirement,
                     resp_get_project, resp_find_project_hook,
                     resp_create_project_hook, resp_delete_project_hook)

# Gitlab module requirements
if python_version_match_requirement():
    from gitlab.v4.objects import ProjectHook

# Unit tests requirements
from httmock import with_httmock  # noqa


class TestGitlabHook(GitlabModuleTestCase):
    def setUp(self):
        super(TestGitlabHook, self).setUp()

        self.moduleUtil = GitLabHook(module=self.mock_module, gitlab_instance=self.gitlab_instance)

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_hook)
    def test_hook_exist(self):
        project = self.gitlab_instance.projects.get(1)

        rvalue = self.moduleUtil.existsHook(project, "http://example.com/hook")

        self.assertEqual(rvalue, True)

        rvalue = self.moduleUtil.existsHook(project, "http://gitlab.com/hook")

        self.assertEqual(rvalue, False)

    @with_httmock(resp_get_project)
    @with_httmock(resp_create_project_hook)
    def test_create_hook(self):
        project = self.gitlab_instance.projects.get(1)

        hook = self.moduleUtil.createHook(project, {"url": "http://example.com/hook"})

        self.assertEqual(type(hook), ProjectHook)
        self.assertEqual(hook.url, "http://example.com/hook")

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_hook)
    def test_update_hook(self):
        project = self.gitlab_instance.projects.get(1)
        hook = self.moduleUtil.findHook(project, "http://example.com/hook")

        changed, newHook = self.moduleUtil.updateHook(hook, {"url": "http://gitlab.com/hook"})

        self.assertEqual(changed, True)
        self.assertEqual(type(newHook), ProjectHook)
        self.assertEqual(newHook.url, "http://gitlab.com/hook")

        changed, newHook = self.moduleUtil.updateHook(hook, {"url": "http://gitlab.com/hook"})

        self.assertEqual(changed, False)
        self.assertEqual(newHook.url, "http://gitlab.com/hook")

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_hook)
    @with_httmock(resp_delete_project_hook)
    def test_delete_hook(self):
        project = self.gitlab_instance.projects.get(1)

        self.moduleUtil.existsHook(project, "http://example.com/hook")

        rvalue = self.moduleUtil.deleteHook()

        self.assertEqual(rvalue, None)



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat.mock import patch
from ansible.modules.network.onyx import onyx_buffer_pool
from units.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxBufferPoolModule(TestOnyxModule):

    module = onyx_buffer_pool
    buffer_pool_configured = False

    def setUp(self):
        super(TestOnyxBufferPoolModule, self).setUp()
        self.mock_get_buffer_pool_config = patch.object(
            onyx_buffer_pool.OnyxBufferPoolModule, "_show_traffic_pool")
        self.get_buffer_pool_config = self.mock_get_buffer_pool_config.start()

        self.mock_load_config = patch(
            'ansible.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxBufferPoolModule, self).tearDown()
        self.mock_get_buffer_pool_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        buffer_pool_config_file = 'onyx_buffer_pool.cfg'
        self.get_buffer_pool_config.return_value = None

        if self.buffer_pool_configured is True:
            buffer_pool_data = load_fixture(buffer_pool_config_file)
            self.get_buffer_pool_config.return_value = buffer_pool_data

        self.load_config.return_value = None

    def test_buffer_pool_no_change(self):
        self.buffer_pool_configured = True
        set_module_args(dict(name="roce", pool_type="lossless",
                             memory_percent=50.0, switch_priority=3))
        self.execute_module(changed=False)

    def test_buffer_pool_with_change(self):
        set_module_args(dict(name="roce", pool_type="lossless",
                             memory_percent=50.0, switch_priority=3))
        commands = ["traffic pool roce type lossless",
                    "traffic pool roce memory percent 50.0",
                    "traffic pool roce map switch-priority 3"
                    ]
        self.execute_module(changed=True, commands=commands)

    def test_memory_percent_with_change(self):
        self.buffer_pool_configured = True
        set_module_args(dict(name="roce", pool_type="lossless",
                             memory_percent=60.0, switch_priority=3))
        commands = ["traffic pool roce memory percent 60.0"]
        self.execute_module(changed=True, commands=commands)

    def test_switch_priority_with_change(self):
        self.buffer_pool_configured = True
        set_module_args(dict(name="roce", pool_type="lossless",
                             memory_percent=50.0, switch_priority=5))
        commands = ["traffic pool roce map switch-priority 5"]
        self.execute_module(changed=True, commands=commands)

    def test_pool_type_with_change(self):
        self.buffer_pool_configured = True
        set_module_args(dict(name="roce", memory_percent=50.0, switch_priority=3))
        commands = ["traffic pool roce type lossy"]
        self.execute_module(changed=True, commands=commands)



# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import json

from units.compat.mock import patch
from ansible.modules.network.netvisor import pn_dscp_map_pri_map
from units.modules.utils import set_module_args
from .nvos_module import TestNvosModule, load_fixture


class TestCpuClassModule(TestNvosModule):

    module = pn_dscp_map_pri_map

    def setUp(self):
        self.mock_run_nvos_commands = patch('ansible.modules.network.netvisor.pn_dscp_map_pri_map.run_cli')
        self.run_nvos_commands = self.mock_run_nvos_commands.start()

        self.mock_run_check_cli = patch('ansible.modules.network.netvisor.pn_dscp_map_pri_map.check_cli')
        self.run_check_cli = self.mock_run_check_cli.start()

    def tearDown(self):
        self.mock_run_nvos_commands.stop()
        self.mock_run_check_cli.stop()

    def run_cli_patch(self, module, cli, state_map):
        if state_map['update'] == 'dscp-map-pri-map-modify':
            results = dict(
                changed=True,
                cli_cmd=cli
            )
        module.exit_json(**results)

    def load_fixtures(self, commands=None, state=None, transport='cli'):
        self.run_nvos_commands.side_effect = self.run_cli_patch
        if state == 'update':
            self.run_check_cli.return_value = True

    def test_dscp_map_pri_map_t1(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn_name': 'foo',
                         'pn_pri': '0', 'pn_dsmap': '40', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 dscp-map-pri-map-modify  pri 0 name foo dsmap 40'
        self.assertEqual(result['cli_cmd'], expected_cmd)

    def test_dscp_map_pri_map_t2(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn_name': 'foo',
                         'pn_pri': '1', 'pn_dsmap': '8,10,12,14', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 dscp-map-pri-map-modify  pri 1 name foo dsmap 8,10,12,14'
        self.assertEqual(result['cli_cmd'], expected_cmd)

    def test_dscp_map_pri_map_t3(self):
        set_module_args({'pn_cliswitch': 'sw01', 'pn_name': 'foo',
                         'pn_pri': '2', 'pn_dsmap': '25', 'state': 'update'})
        result = self.execute_module(changed=True, state='update')
        expected_cmd = ' switch sw01 dscp-map-pri-map-modify  pri 2 name foo dsmap 25'
        self.assertEqual(result['cli_cmd'], expected_cmd)



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_settings
short_description: Configure VDOM settings in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure system feature and settings category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip adress.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    system_settings:
        description:
            - Configure VDOM settings.
        default: null
        suboptions:
            allow-subnet-overlap:
                description:
                    - Enable/disable allowing interface subnets to use overlapping IP addresses.
                choices:
                    - enable
                    - disable
            asymroute:
                description:
                    - Enable/disable IPv4 asymmetric routing.
                choices:
                    - enable
                    - disable
            asymroute-icmp:
                description:
                    - Enable/disable ICMP asymmetric routing.
                choices:
                    - enable
                    - disable
            asymroute6:
                description:
                    - Enable/disable asymmetric IPv6 routing.
                choices:
                    - enable
                    - disable
            asymroute6-icmp:
                description:
                    - Enable/disable asymmetric ICMPv6 routing.
                choices:
                    - enable
                    - disable
            bfd:
                description:
                    - Enable/disable Bi-directional Forwarding Detection (BFD) on all interfaces.
                choices:
                    - enable
                    - disable
            bfd-desired-min-tx:
                description:
                    - BFD desired minimal transmit interval (1 - 100000 ms, default = 50).
            bfd-detect-mult:
                description:
                    - BFD detection multiplier (1 - 50, default = 3).
            bfd-dont-enforce-src-port:
                description:
                    - Enable to not enforce verifying the source port of BFD Packets.
                choices:
                    - enable
                    - disable
            bfd-required-min-rx:
                description:
                    - BFD required minimal receive interval (1 - 100000 ms, default = 50).
            block-land-attack:
                description:
                    - Enable/disable blocking of land attacks.
                choices:
                    - disable
                    - enable
            central-nat:
                description:
                    - Enable/disable central NAT.
                choices:
                    - enable
                    - disable
            comments:
                description:
                    - VDOM comments.
            compliance-check:
                description:
                    - Enable/disable PCI DSS compliance checking.
                choices:
                    - enable
                    - disable
            default-voip-alg-mode:
                description:
                    - Configure how the FortiGate handles VoIP traffic when a policy that accepts the traffic doesn't include a VoIP profile.
                choices:
                    - proxy-based
                    - kernel-helper-based
            deny-tcp-with-icmp:
                description:
                    - Enable/disable denying TCP by sending an ICMP communication prohibited packet.
                choices:
                    - enable
                    - disable
            device:
                description:
                    - Interface to use for management access for NAT mode. Source system.interface.name.
            dhcp-proxy:
                description:
                    - Enable/disable the DHCP Proxy.
                choices:
                    - enable
                    - disable
            dhcp-server-ip:
                description:
                    - DHCP Server IPv4 address.
            dhcp6-server-ip:
                description:
                    - DHCPv6 server IPv6 address.
            discovered-device-timeout:
                description:
                    - Timeout for discovered devices (1 - 365 days, default = 28).
            ecmp-max-paths:
                description:
                    - Maximum number of Equal Cost Multi-Path (ECMP) next-hops. Set to 1 to disable ECMP routing (1 - 100, default = 10).
            email-portal-check-dns:
                description:
                    - Enable/disable using DNS to validate email addresses collected by a captive portal.
                choices:
                    - disable
                    - enable
            firewall-session-dirty:
                description:
                    - Select how to manage sessions affected by firewall policy configuration changes.
                choices:
                    - check-all
                    - check-new
                    - check-policy-option
            fw-session-hairpin:
                description:
                    - Enable/disable checking for a matching policy each time hairpin traffic goes through the FortiGate.
                choices:
                    - enable
                    - disable
            gateway:
                description:
                    - Transparent mode IPv4 default gateway IP address.
            gateway6:
                description:
                    - Transparent mode IPv4 default gateway IP address.
            gui-advanced-policy:
                description:
                    - Enable/disable advanced policy configuration on the GUI.
                choices:
                    - enable
                    - disable
            gui-allow-unnamed-policy:
                description:
                    - Enable/disable the requirement for policy naming on the GUI.
                choices:
                    - enable
                    - disable
            gui-antivirus:
                description:
                    - Enable/disable AntiVirus on the GUI.
                choices:
                    - enable
                    - disable
            gui-ap-profile:
                description:
                    - Enable/disable FortiAP profiles on the GUI.
                choices:
                    - enable
                    - disable
            gui-application-control:
                description:
                    - Enable/disable application control on the GUI.
                choices:
                    - enable
                    - disable
            gui-default-policy-columns:
                description:
                    - Default columns to display for policy lists on GUI.
                suboptions:
                    name:
                        description:
                            - Select column name.
                        required: true
            gui-dhcp-advanced:
                description:
                    - Enable/disable advanced DHCP options on the GUI.
                choices:
                    - enable
                    - disable
            gui-dlp:
                description:
                    - Enable/disable DLP on the GUI.
                choices:
                    - enable
                    - disable
            gui-dns-database:
                description:
                    - Enable/disable DNS database settings on the GUI.
                choices:
                    - enable
                    - disable
            gui-dnsfilter:
                description:
                    - Enable/disable DNS Filtering on the GUI.
                choices:
                    - enable
                    - disable
            gui-domain-ip-reputation:
                description:
                    - Enable/disable Domain and IP Reputation on the GUI.
                choices:
                    - enable
                    - disable
            gui-dos-policy:
                description:
                    - Enable/disable DoS policies on the GUI.
                choices:
                    - enable
                    - disable
            gui-dynamic-profile-display:
                description:
                    - Enable/disable RADIUS Single Sign On (RSSO) on the GUI.
                choices:
                    - enable
                    - disable
            gui-dynamic-routing:
                description:
                    - Enable/disable dynamic routing on the GUI.
                choices:
                    - enable
                    - disable
            gui-email-collection:
                description:
                    - Enable/disable email collection on the GUI.
                choices:
                    - enable
                    - disable
            gui-endpoint-control:
                description:
                    - Enable/disable endpoint control on the GUI.
                choices:
                    - enable
                    - disable
            gui-endpoint-control-advanced:
                description:
                    - Enable/disable advanced endpoint control options on the GUI.
                choices:
                    - enable
                    - disable
            gui-explicit-proxy:
                description:
                    - Enable/disable the explicit proxy on the GUI.
                choices:
                    - enable
                    - disable
            gui-fortiap-split-tunneling:
                description:
                    - Enable/disable FortiAP split tunneling on the GUI.
                choices:
                    - enable
                    - disable
            gui-fortiextender-controller:
                description:
                    - Enable/disable FortiExtender on the GUI.
                choices:
                    - enable
                    - disable
            gui-icap:
                description:
                    - Enable/disable ICAP on the GUI.
                choices:
                    - enable
                    - disable
            gui-implicit-policy:
                description:
                    - Enable/disable implicit firewall policies on the GUI.
                choices:
                    - enable
                    - disable
            gui-ips:
                description:
                    - Enable/disable IPS on the GUI.
                choices:
                    - enable
                    - disable
            gui-load-balance:
                description:
                    - Enable/disable server load balancing on the GUI.
                choices:
                    - enable
                    - disable
            gui-local-in-policy:
                description:
                    - Enable/disable Local-In policies on the GUI.
                choices:
                    - enable
                    - disable
            gui-local-reports:
                description:
                    - Enable/disable local reports on the GUI.
                choices:
                    - enable
                    - disable
            gui-multicast-policy:
                description:
                    - Enable/disable multicast firewall policies on the GUI.
                choices:
                    - enable
                    - disable
            gui-multiple-interface-policy:
                description:
                    - Enable/disable adding multiple interfaces to a policy on the GUI.
                choices:
                    - enable
                    - disable
            gui-multiple-utm-profiles:
                description:
                    - Enable/disable multiple UTM profiles on the GUI.
                choices:
                    - enable
                    - disable
            gui-nat46-64:
                description:
                    - Enable/disable NAT46 and NAT64 settings on the GUI.
                choices:
                    - enable
                    - disable
            gui-object-colors:
                description:
                    - Enable/disable object colors on the GUI.
                choices:
                    - enable
                    - disable
            gui-policy-based-ipsec:
                description:
                    - Enable/disable policy-based IPsec VPN on the GUI.
                choices:
                    - enable
                    - disable
            gui-policy-learning:
                description:
                    - Enable/disable firewall policy learning mode on the GUI.
                choices:
                    - enable
                    - disable
            gui-replacement-message-groups:
                description:
                    - Enable/disable replacement message groups on the GUI.
                choices:
                    - enable
                    - disable
            gui-spamfilter:
                description:
                    - Enable/disable Antispam on the GUI.
                choices:
                    - enable
                    - disable
            gui-sslvpn-personal-bookmarks:
                description:
                    - Enable/disable SSL-VPN personal bookmark management on the GUI.
                choices:
                    - enable
                    - disable
            gui-sslvpn-realms:
                description:
                    - Enable/disable SSL-VPN realms on the GUI.
                choices:
                    - enable
                    - disable
            gui-switch-controller:
                description:
                    - Enable/disable the switch controller on the GUI.
                choices:
                    - enable
                    - disable
            gui-threat-weight:
                description:
                    - Enable/disable threat weight on the GUI.
                choices:
                    - enable
                    - disable
            gui-traffic-shaping:
                description:
                    - Enable/disable traffic shaping on the GUI.
                choices:
                    - enable
                    - disable
            gui-voip-profile:
                description:
                    - Enable/disable VoIP profiles on the GUI.
                choices:
                    - enable
                    - disable
            gui-vpn:
                description:
                    - Enable/disable VPN tunnels on the GUI.
                choices:
                    - enable
                    - disable
            gui-waf-profile:
                description:
                    - Enable/disable Web Application Firewall on the GUI.
                choices:
                    - enable
                    - disable
            gui-wan-load-balancing:
                description:
                    - Enable/disable SD-WAN on the GUI.
                choices:
                    - enable
                    - disable
            gui-wanopt-cache:
                description:
                    - Enable/disable WAN Optimization and Web Caching on the GUI.
                choices:
                    - enable
                    - disable
            gui-webfilter:
                description:
                    - Enable/disable Web filtering on the GUI.
                choices:
                    - enable
                    - disable
            gui-webfilter-advanced:
                description:
                    - Enable/disable advanced web filtering on the GUI.
                choices:
                    - enable
                    - disable
            gui-wireless-controller:
                description:
                    - Enable/disable the wireless controller on the GUI.
                choices:
                    - enable
                    - disable
            http-external-dest:
                description:
                    - Offload HTTP traffic to FortiWeb or FortiCache.
                choices:
                    - fortiweb
                    - forticache
            ike-dn-format:
                description:
                    - Configure IKE ASN.1 Distinguished Name format conventions.
                choices:
                    - with-space
                    - no-space
            ike-quick-crash-detect:
                description:
                    - Enable/disable IKE quick crash detection (RFC 6290).
                choices:
                    - enable
                    - disable
            ike-session-resume:
                description:
                    - Enable/disable IKEv2 session resumption (RFC 5723).
                choices:
                    - enable
                    - disable
            implicit-allow-dns:
                description:
                    - Enable/disable implicitly allowing DNS traffic.
                choices:
                    - enable
                    - disable
            inspection-mode:
                description:
                    - Inspection mode (proxy-based or flow-based).
                choices:
                    - proxy
                    - flow
            ip:
                description:
                    - IP address and netmask.
            ip6:
                description:
                    - IPv6 address prefix for NAT mode.
            link-down-access:
                description:
                    - Enable/disable link down access traffic.
                choices:
                    - enable
                    - disable
            lldp-transmission:
                description:
                    - Enable/disable Link Layer Discovery Protocol (LLDP) for this VDOM or apply global settings to this VDOM.
                choices:
                    - enable
                    - disable
                    - global
            mac-ttl:
                description:
                    - Duration of MAC addresses in Transparent mode (300 - 8640000 sec, default = 300).
            manageip:
                description:
                    - Transparent mode IPv4 management IP address and netmask.
            manageip6:
                description:
                    - Transparent mode IPv6 management IP address and netmask.
            multicast-forward:
                description:
                    - Enable/disable multicast forwarding.
                choices:
                    - enable
                    - disable
            multicast-skip-policy:
                description:
                    - Enable/disable allowing multicast traffic through the FortiGate without a policy check.
                choices:
                    - enable
                    - disable
            multicast-ttl-notchange:
                description:
                    - Enable/disable preventing the FortiGate from changing the TTL for forwarded multicast packets.
                choices:
                    - enable
                    - disable
            ngfw-mode:
                description:
                    - Next Generation Firewall (NGFW) mode.
                choices:
                    - profile-based
                    - policy-based
            opmode:
                description:
                    - Firewall operation mode (NAT or Transparent).
                choices:
                    - nat
                    - transparent
            prp-trailer-action:
                description:
                    - Enable/disable action to take on PRP trailer.
                choices:
                    - enable
                    - disable
            sccp-port:
                description:
                    - TCP port the SCCP proxy monitors for SCCP traffic (0 - 65535, default = 2000).
            ses-denied-traffic:
                description:
                    - Enable/disable including denied session in the session table.
                choices:
                    - enable
                    - disable
            sip-helper:
                description:
                    - Enable/disable the SIP session helper to process SIP sessions unless SIP sessions are accepted by the SIP application layer gateway
                       (ALG).
                choices:
                    - enable
                    - disable
            sip-nat-trace:
                description:
                    - Enable/disable recording the original SIP source IP address when NAT is used.
                choices:
                    - enable
                    - disable
            sip-ssl-port:
                description:
                    - TCP port the SIP proxy monitors for SIP SSL/TLS traffic (0 - 65535, default = 5061).
            sip-tcp-port:
                description:
                    - TCP port the SIP proxy monitors for SIP traffic (0 - 65535, default = 5060).
            sip-udp-port:
                description:
                    - UDP port the SIP proxy monitors for SIP traffic (0 - 65535, default = 5060).
            snat-hairpin-traffic:
                description:
                    - Enable/disable source NAT (SNAT) for hairpin traffic.
                choices:
                    - enable
                    - disable
            ssl-ssh-profile:
                description:
                    - Profile for SSL/SSH inspection. Source firewall.ssl-ssh-profile.name.
            status:
                description:
                    - Enable/disable this VDOM.
                choices:
                    - enable
                    - disable
            strict-src-check:
                description:
                    - Enable/disable strict source verification.
                choices:
                    - enable
                    - disable
            tcp-session-without-syn:
                description:
                    - Enable/disable allowing TCP session without SYN flags.
                choices:
                    - enable
                    - disable
            utf8-spam-tagging:
                description:
                    - Enable/disable converting antispam tags to UTF-8 for better non-ASCII character support.
                choices:
                    - enable
                    - disable
            v4-ecmp-mode:
                description:
                    - IPv4 Equal-cost multi-path (ECMP) routing and load balancing mode.
                choices:
                    - source-ip-based
                    - weight-based
                    - usage-based
                    - source-dest-ip-based
            vpn-stats-log:
                description:
                    - Enable/disable periodic VPN log statistics for one or more types of VPN. Separate names with a space.
                choices:
                    - ipsec
                    - pptp
                    - l2tp
                    - ssl
            vpn-stats-period:
                description:
                    - Period to send VPN log statistics (60 - 86400 sec).
            wccp-cache-engine:
                description:
                    - Enable/disable WCCP cache engine.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure VDOM settings.
    fortios_system_settings:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      system_settings:
        allow-subnet-overlap: "enable"
        asymroute: "enable"
        asymroute-icmp: "enable"
        asymroute6: "enable"
        asymroute6-icmp: "enable"
        bfd: "enable"
        bfd-desired-min-tx: "9"
        bfd-detect-mult: "10"
        bfd-dont-enforce-src-port: "enable"
        bfd-required-min-rx: "12"
        block-land-attack: "disable"
        central-nat: "enable"
        comments: "<your_own_value>"
        compliance-check: "enable"
        default-voip-alg-mode: "proxy-based"
        deny-tcp-with-icmp: "enable"
        device: "<your_own_value> (source system.interface.name)"
        dhcp-proxy: "enable"
        dhcp-server-ip: "<your_own_value>"
        dhcp6-server-ip: "<your_own_value>"
        discovered-device-timeout: "23"
        ecmp-max-paths: "24"
        email-portal-check-dns: "disable"
        firewall-session-dirty: "check-all"
        fw-session-hairpin: "enable"
        gateway: "<your_own_value>"
        gateway6: "<your_own_value>"
        gui-advanced-policy: "enable"
        gui-allow-unnamed-policy: "enable"
        gui-antivirus: "enable"
        gui-ap-profile: "enable"
        gui-application-control: "enable"
        gui-default-policy-columns:
         -
            name: "default_name_36"
        gui-dhcp-advanced: "enable"
        gui-dlp: "enable"
        gui-dns-database: "enable"
        gui-dnsfilter: "enable"
        gui-domain-ip-reputation: "enable"
        gui-dos-policy: "enable"
        gui-dynamic-profile-display: "enable"
        gui-dynamic-routing: "enable"
        gui-email-collection: "enable"
        gui-endpoint-control: "enable"
        gui-endpoint-control-advanced: "enable"
        gui-explicit-proxy: "enable"
        gui-fortiap-split-tunneling: "enable"
        gui-fortiextender-controller: "enable"
        gui-icap: "enable"
        gui-implicit-policy: "enable"
        gui-ips: "enable"
        gui-load-balance: "enable"
        gui-local-in-policy: "enable"
        gui-local-reports: "enable"
        gui-multicast-policy: "enable"
        gui-multiple-interface-policy: "enable"
        gui-multiple-utm-profiles: "enable"
        gui-nat46-64: "enable"
        gui-object-colors: "enable"
        gui-policy-based-ipsec: "enable"
        gui-policy-learning: "enable"
        gui-replacement-message-groups: "enable"
        gui-spamfilter: "enable"
        gui-sslvpn-personal-bookmarks: "enable"
        gui-sslvpn-realms: "enable"
        gui-switch-controller: "enable"
        gui-threat-weight: "enable"
        gui-traffic-shaping: "enable"
        gui-voip-profile: "enable"
        gui-vpn: "enable"
        gui-waf-profile: "enable"
        gui-wan-load-balancing: "enable"
        gui-wanopt-cache: "enable"
        gui-webfilter: "enable"
        gui-webfilter-advanced: "enable"
        gui-wireless-controller: "enable"
        http-external-dest: "fortiweb"
        ike-dn-format: "with-space"
        ike-quick-crash-detect: "enable"
        ike-session-resume: "enable"
        implicit-allow-dns: "enable"
        inspection-mode: "proxy"
        ip: "<your_own_value>"
        ip6: "<your_own_value>"
        link-down-access: "enable"
        lldp-transmission: "enable"
        mac-ttl: "89"
        manageip: "<your_own_value>"
        manageip6: "<your_own_value>"
        multicast-forward: "enable"
        multicast-skip-policy: "enable"
        multicast-ttl-notchange: "enable"
        ngfw-mode: "profile-based"
        opmode: "nat"
        prp-trailer-action: "enable"
        sccp-port: "98"
        ses-denied-traffic: "enable"
        sip-helper: "enable"
        sip-nat-trace: "enable"
        sip-ssl-port: "102"
        sip-tcp-port: "103"
        sip-udp-port: "104"
        snat-hairpin-traffic: "enable"
        ssl-ssh-profile: "<your_own_value> (source firewall.ssl-ssh-profile.name)"
        status: "enable"
        strict-src-check: "enable"
        tcp-session-without-syn: "enable"
        utf8-spam-tagging: "enable"
        v4-ecmp-mode: "source-ip-based"
        vpn-stats-log: "ipsec"
        vpn-stats-period: "113"
        wccp-cache-engine: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_settings_data(json):
    option_list = ['allow-subnet-overlap', 'asymroute', 'asymroute-icmp',
                   'asymroute6', 'asymroute6-icmp', 'bfd',
                   'bfd-desired-min-tx', 'bfd-detect-mult', 'bfd-dont-enforce-src-port',
                   'bfd-required-min-rx', 'block-land-attack', 'central-nat',
                   'comments', 'compliance-check', 'default-voip-alg-mode',
                   'deny-tcp-with-icmp', 'device', 'dhcp-proxy',
                   'dhcp-server-ip', 'dhcp6-server-ip', 'discovered-device-timeout',
                   'ecmp-max-paths', 'email-portal-check-dns', 'firewall-session-dirty',
                   'fw-session-hairpin', 'gateway', 'gateway6',
                   'gui-advanced-policy', 'gui-allow-unnamed-policy', 'gui-antivirus',
                   'gui-ap-profile', 'gui-application-control', 'gui-default-policy-columns',
                   'gui-dhcp-advanced', 'gui-dlp', 'gui-dns-database',
                   'gui-dnsfilter', 'gui-domain-ip-reputation', 'gui-dos-policy',
                   'gui-dynamic-profile-display', 'gui-dynamic-routing', 'gui-email-collection',
                   'gui-endpoint-control', 'gui-endpoint-control-advanced', 'gui-explicit-proxy',
                   'gui-fortiap-split-tunneling', 'gui-fortiextender-controller', 'gui-icap',
                   'gui-implicit-policy', 'gui-ips', 'gui-load-balance',
                   'gui-local-in-policy', 'gui-local-reports', 'gui-multicast-policy',
                   'gui-multiple-interface-policy', 'gui-multiple-utm-profiles', 'gui-nat46-64',
                   'gui-object-colors', 'gui-policy-based-ipsec', 'gui-policy-learning',
                   'gui-replacement-message-groups', 'gui-spamfilter', 'gui-sslvpn-personal-bookmarks',
                   'gui-sslvpn-realms', 'gui-switch-controller', 'gui-threat-weight',
                   'gui-traffic-shaping', 'gui-voip-profile', 'gui-vpn',
                   'gui-waf-profile', 'gui-wan-load-balancing', 'gui-wanopt-cache',
                   'gui-webfilter', 'gui-webfilter-advanced', 'gui-wireless-controller',
                   'http-external-dest', 'ike-dn-format', 'ike-quick-crash-detect',
                   'ike-session-resume', 'implicit-allow-dns', 'inspection-mode',
                   'ip', 'ip6', 'link-down-access',
                   'lldp-transmission', 'mac-ttl', 'manageip',
                   'manageip6', 'multicast-forward', 'multicast-skip-policy',
                   'multicast-ttl-notchange', 'ngfw-mode', 'opmode',
                   'prp-trailer-action', 'sccp-port', 'ses-denied-traffic',
                   'sip-helper', 'sip-nat-trace', 'sip-ssl-port',
                   'sip-tcp-port', 'sip-udp-port', 'snat-hairpin-traffic',
                   'ssl-ssh-profile', 'status', 'strict-src-check',
                   'tcp-session-without-syn', 'utf8-spam-tagging', 'v4-ecmp-mode',
                   'vpn-stats-log', 'vpn-stats-period', 'wccp-cache-engine']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def system_settings(data, fos):
    vdom = data['vdom']
    system_settings_data = data['system_settings']
    filtered_data = filter_system_settings_data(system_settings_data)
    return fos.set('system',
                   'settings',
                   data=filtered_data,
                   vdom=vdom)


def fortios_system(data, fos):
    login(data)

    methodlist = ['system_settings']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "system_settings": {
            "required": False, "type": "dict",
            "options": {
                "allow-subnet-overlap": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "asymroute": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "asymroute-icmp": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "asymroute6": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "asymroute6-icmp": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "bfd": {"required": False, "type": "str",
                        "choices": ["enable", "disable"]},
                "bfd-desired-min-tx": {"required": False, "type": "int"},
                "bfd-detect-mult": {"required": False, "type": "int"},
                "bfd-dont-enforce-src-port": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "bfd-required-min-rx": {"required": False, "type": "int"},
                "block-land-attack": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "central-nat": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "comments": {"required": False, "type": "str"},
                "compliance-check": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "default-voip-alg-mode": {"required": False, "type": "str",
                                          "choices": ["proxy-based", "kernel-helper-based"]},
                "deny-tcp-with-icmp": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "device": {"required": False, "type": "str"},
                "dhcp-proxy": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "dhcp-server-ip": {"required": False, "type": "str"},
                "dhcp6-server-ip": {"required": False, "type": "str"},
                "discovered-device-timeout": {"required": False, "type": "int"},
                "ecmp-max-paths": {"required": False, "type": "int"},
                "email-portal-check-dns": {"required": False, "type": "str",
                                           "choices": ["disable", "enable"]},
                "firewall-session-dirty": {"required": False, "type": "str",
                                           "choices": ["check-all", "check-new", "check-policy-option"]},
                "fw-session-hairpin": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "gateway": {"required": False, "type": "str"},
                "gateway6": {"required": False, "type": "str"},
                "gui-advanced-policy": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-allow-unnamed-policy": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "gui-antivirus": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "gui-ap-profile": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "gui-application-control": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "gui-default-policy-columns": {"required": False, "type": "list",
                                               "options": {
                                                   "name": {"required": True, "type": "str"}
                                               }},
                "gui-dhcp-advanced": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "gui-dlp": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "gui-dns-database": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "gui-dnsfilter": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "gui-domain-ip-reputation": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                "gui-dos-policy": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "gui-dynamic-profile-display": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "gui-dynamic-routing": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-email-collection": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "gui-endpoint-control": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "gui-endpoint-control-advanced": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "gui-explicit-proxy": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "gui-fortiap-split-tunneling": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "gui-fortiextender-controller": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "gui-icap": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "gui-implicit-policy": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-ips": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "gui-load-balance": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "gui-local-in-policy": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-local-reports": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "gui-multicast-policy": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "gui-multiple-interface-policy": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "gui-multiple-utm-profiles": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                "gui-nat46-64": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "gui-object-colors": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "gui-policy-based-ipsec": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "gui-policy-learning": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-replacement-message-groups": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                "gui-spamfilter": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "gui-sslvpn-personal-bookmarks": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "gui-sslvpn-realms": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "gui-switch-controller": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "gui-threat-weight": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "gui-traffic-shaping": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "gui-voip-profile": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "gui-vpn": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "gui-waf-profile": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "gui-wan-load-balancing": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "gui-wanopt-cache": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "gui-webfilter": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "gui-webfilter-advanced": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "gui-wireless-controller": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "http-external-dest": {"required": False, "type": "str",
                                       "choices": ["fortiweb", "forticache"]},
                "ike-dn-format": {"required": False, "type": "str",
                                  "choices": ["with-space", "no-space"]},
                "ike-quick-crash-detect": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "ike-session-resume": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "implicit-allow-dns": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "inspection-mode": {"required": False, "type": "str",
                                    "choices": ["proxy", "flow"]},
                "ip": {"required": False, "type": "str"},
                "ip6": {"required": False, "type": "str"},
                "link-down-access": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "lldp-transmission": {"required": False, "type": "str",
                                      "choices": ["enable", "disable", "global"]},
                "mac-ttl": {"required": False, "type": "int"},
                "manageip": {"required": False, "type": "str"},
                "manageip6": {"required": False, "type": "str"},
                "multicast-forward": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "multicast-skip-policy": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "multicast-ttl-notchange": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "ngfw-mode": {"required": False, "type": "str",
                              "choices": ["profile-based", "policy-based"]},
                "opmode": {"required": False, "type": "str",
                           "choices": ["nat", "transparent"]},
                "prp-trailer-action": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "sccp-port": {"required": False, "type": "int"},
                "ses-denied-traffic": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "sip-helper": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "sip-nat-trace": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "sip-ssl-port": {"required": False, "type": "int"},
                "sip-tcp-port": {"required": False, "type": "int"},
                "sip-udp-port": {"required": False, "type": "int"},
                "snat-hairpin-traffic": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "ssl-ssh-profile": {"required": False, "type": "str"},
                "status": {"required": False, "type": "str",
                           "choices": ["enable", "disable"]},
                "strict-src-check": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "tcp-session-without-syn": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "utf8-spam-tagging": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "v4-ecmp-mode": {"required": False, "type": "str",
                                 "choices": ["source-ip-based", "weight-based", "usage-based",
                                             "source-dest-ip-based"]},
                "vpn-stats-log": {"required": False, "type": "str",
                                  "choices": ["ipsec", "pptp", "l2tp",
                                              "ssl"]},
                "vpn-stats-period": {"required": False, "type": "int"},
                "wccp-cache-engine": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


from ansible.modules.net_tools.nios import nios_ptr_record
from ansible.module_utils.net_tools.nios import api
from units.compat.mock import patch, MagicMock, Mock
from .test_nios_module import TestNiosModule, load_fixture


class TestNiosPTRRecordModule(TestNiosModule):

    module = nios_ptr_record

    def setUp(self):

        super(TestNiosPTRRecordModule, self).setUp()
        self.module = MagicMock(name='ansible.modules.net_tools.nios.nios_ptr_record.WapiModule')
        self.module.check_mode = False
        self.module.params = {'provider': None}

        self.mock_wapi = patch('ansible.modules.net_tools.nios.nios_ptr_record.WapiModule')
        self.exec_command = self.mock_wapi.start()
        self.mock_wapi_run = patch('ansible.modules.net_tools.nios.nios_ptr_record.WapiModule.run')
        self.mock_wapi_run.start()

        self.load_config = self.mock_wapi_run.start()

    def tearDown(self):
        super(TestNiosPTRRecordModule, self).tearDown()
        self.mock_wapi.stop()

    def _get_wapi(self, test_object):
        wapi = api.WapiModule(self.module)
        wapi.get_object = Mock(name='get_object', return_value=test_object)
        wapi.create_object = Mock(name='create_object')
        wapi.update_object = Mock(name='update_object')
        wapi.delete_object = Mock(name='delete_object')
        return wapi

    def load_fixtures(self, commands=None):
        self.exec_command.return_value = (0, load_fixture('nios_result.txt').strip(), None)
        self.load_config.return_value = dict(diff=None, session='session')

    def test_nios_ptr_record_create(self):
        self.module.params = {'provider': None, 'state': 'present', 'ptrdname': 'ansible.test.com',
                              'ipv4addr': '10.36.241.14', 'comment': None, 'extattrs': None}

        test_object = None
        test_spec = {
            "ipv4addr": {"ib_req": True},
            "ptrdname": {"ib_req": True},
            "comment": {},
            "extattrs": {}
        }

        wapi = self._get_wapi(test_object)
        print("WAPI: ", wapi)
        res = wapi.run('testobject', test_spec)

        self.assertTrue(res['changed'])
        wapi.create_object.assert_called_once_with('testobject', {'ipv4addr': '10.36.241.14', 'ptrdname': 'ansible.test.com'})

    def test_nios_ptr_record_remove(self):
        self.module.params = {'provider': None, 'state': 'absent', 'ptrdname': 'ansible.test.com',
                              'ipv4addr': '10.36.241.14', 'comment': None, 'extattrs': None}

        ref = "record:ptr/ZG5zLm5ldHdvcmtfdmlldyQw:14.241.36.10.in-addr.arpa/default"

        test_object = [{
            "comment": "test comment",
            "_ref": ref,
            "ptrdname": "ansible.test.com",
            "ipv4addr": "10.36.241.14",
            "extattrs": {'Site': {'value': 'test'}}
        }]

        test_spec = {
            "ipv4addr": {"ib_req": True},
            "ptrdname": {"ib_req": True},
            "comment": {},
            "extattrs": {}
        }

        wapi = self._get_wapi(test_object)
        res = wapi.run('testobject', test_spec)
        self.assertTrue(res['changed'])
        wapi.delete_object.assert_called_once_with(ref)

    def test_nios_ptr_record_update_comment(self):
        self.module.params = {'provider': None, 'state': 'present', 'ptrdname': 'ansible.test.com',
                              'ipv4addr': '10.36.241.14', 'comment': 'updated comment', 'extattrs': None}

        test_object = [
            {
                "comment": "test comment",
                "_ref": "record:ptr/ZG5zLm5ldHdvcmtfdmlldyQw:14.241.36.10.in-addr.arpa/default",
                "ptrdname": "ansible.test.com",
                "ipv4addr": "10.36.241.14",
                "extattrs": {}
            }
        ]

        test_spec = {
            "ipv4addr": {"ib_req": True},
            "ptrdname": {"ib_req": True},
            "comment": {},
            "extattrs": {}
        }

        wapi = self._get_wapi(test_object)
        res = wapi.run('testobject', test_spec)

        self.assertTrue(res['changed'])
        wapi.update_object.called_once_with(test_object)

    def test_nios_ptr_record_update_record_ptrdname(self):
        self.module.params = {'provider': None, 'state': 'present', 'ptrdname': 'ansible.test.org',
                              'ipv4addr': '10.36.241.14', 'comment': 'comment', 'extattrs': None}

        test_object = [
            {
                "comment": "test comment",
                "_ref": "record:ptr/ZG5zLm5ldHdvcmtfdmlldyQw:14.241.36.10.in-addr.arpa/default",
                "ptrdname": "ansible.test.com",
                "ipv4addr": "10.36.241.14",
                "extattrs": {}
            }
        ]

        test_spec = {
            "ipv4addr": {"ib_req": True},
            "ptrdname": {"ib_req": True},
            "comment": {},
            "extattrs": {}
        }

        wapi = self._get_wapi(test_object)
        res = wapi.run('testobject', test_spec)

        self.assertTrue(res['changed'])
        wapi.update_object.called_once_with(test_object)

    def test_nios_ptr6_record_create(self):
        self.module.params = {'provider': None, 'state': 'present', 'ptrdname': 'ansible6.test.com',
                              'ipv6addr': '2002:8ac3:802d:1242:20d:60ff:fe38:6d16', 'comment': None, 'extattrs': None}

        test_object = None
        test_spec = {"ipv6addr": {"ib_req": True},
                     "ptrdname": {"ib_req": True},
                     "comment": {},
                     "extattrs": {}}

        wapi = self._get_wapi(test_object)
        print("WAPI: ", wapi)
        res = wapi.run('testobject', test_spec)

        self.assertTrue(res['changed'])
        wapi.create_object.assert_called_once_with('testobject', {'ipv6addr': '2002:8ac3:802d:1242:20d:60ff:fe38:6d16', 'ptrdname': 'ansible6.test.com'})



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_antivirus_profile
short_description: Configure AntiVirus profiles in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure antivirus feature and profile category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    antivirus_profile:
        description:
            - Configure AntiVirus profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            analytics-bl-filetype:
                description:
                    - Only submit files matching this DLP file-pattern to FortiSandbox. Source dlp.filepattern.id.
            analytics-db:
                description:
                    - Enable/disable using the FortiSandbox signature database to supplement the AV signature databases.
                choices:
                    - disable
                    - enable
            analytics-max-upload:
                description:
                    - Maximum size of files that can be uploaded to FortiSandbox (1 - 395 MBytes, default = 10).
            analytics-wl-filetype:
                description:
                    - Do not submit files matching this DLP file-pattern to FortiSandbox. Source dlp.filepattern.id.
            av-block-log:
                description:
                    - Enable/disable logging for AntiVirus file blocking.
                choices:
                    - enable
                    - disable
            av-virus-log:
                description:
                    - Enable/disable AntiVirus logging.
                choices:
                    - enable
                    - disable
            comment:
                description:
                    - Comment.
            content-disarm:
                description:
                    - AV Content Disarm and Reconstruction settings.
                suboptions:
                    cover-page:
                        description:
                            - Enable/disable inserting a cover page into the disarmed document.
                        choices:
                            - disable
                            - enable
                    detect-only:
                        description:
                            - Enable/disable only detect disarmable files, do not alter content.
                        choices:
                            - disable
                            - enable
                    office-embed:
                        description:
                            - Enable/disable stripping of embedded objects in Microsoft Office documents.
                        choices:
                            - disable
                            - enable
                    office-hylink:
                        description:
                            - Enable/disable stripping of hyperlinks in Microsoft Office documents.
                        choices:
                            - disable
                            - enable
                    office-linked:
                        description:
                            - Enable/disable stripping of linked objects in Microsoft Office documents.
                        choices:
                            - disable
                            - enable
                    office-macro:
                        description:
                            - Enable/disable stripping of macros in Microsoft Office documents.
                        choices:
                            - disable
                            - enable
                    original-file-destination:
                        description:
                            - Destination to send original file if active content is removed.
                        choices:
                            - fortisandbox
                            - quarantine
                            - discard
                    pdf-act-form:
                        description:
                            - Enable/disable stripping of actions that submit data to other targets in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-act-gotor:
                        description:
                            - Enable/disable stripping of links to other PDFs in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-act-java:
                        description:
                            - Enable/disable stripping of actions that execute JavaScript code in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-act-launch:
                        description:
                            - Enable/disable stripping of links to external applications in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-act-movie:
                        description:
                            - Enable/disable stripping of embedded movies in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-act-sound:
                        description:
                            - Enable/disable stripping of embedded sound files in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-embedfile:
                        description:
                            - Enable/disable stripping of embedded files in PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-hyperlink:
                        description:
                            - Enable/disable stripping of hyperlinks from PDF documents.
                        choices:
                            - disable
                            - enable
                    pdf-javacode:
                        description:
                            - Enable/disable stripping of JavaScript code in PDF documents.
                        choices:
                            - disable
                            - enable
            extended-log:
                description:
                    - Enable/disable extended logging for antivirus.
                choices:
                    - enable
                    - disable
            ftgd-analytics:
                description:
                    - Settings to control which files are uploaded to FortiSandbox.
                choices:
                    - disable
                    - suspicious
                    - everything
            ftp:
                description:
                    - Configure FTP AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    options:
                        description:
                            - Enable/disable FTP AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            http:
                description:
                    - Configure HTTP AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    content-disarm:
                        description:
                            - Enable Content Disarm and Reconstruction for this protocol.
                        choices:
                            - disable
                            - enable
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    options:
                        description:
                            - Enable/disable HTTP AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            imap:
                description:
                    - Configure IMAP AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    content-disarm:
                        description:
                            - Enable Content Disarm and Reconstruction for this protocol.
                        choices:
                            - disable
                            - enable
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    executables:
                        description:
                            - Treat Windows executable files as viruses for the purpose of blocking or monitoring.
                        choices:
                            - default
                            - virus
                    options:
                        description:
                            - Enable/disable IMAP AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            inspection-mode:
                description:
                    - Inspection mode.
                choices:
                    - proxy
                    - flow-based
            mapi:
                description:
                    - Configure MAPI AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    executables:
                        description:
                            - Treat Windows executable files as viruses for the purpose of blocking or monitoring.
                        choices:
                            - default
                            - virus
                    options:
                        description:
                            - Enable/disable MAPI AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            mobile-malware-db:
                description:
                    - Enable/disable using the mobile malware signature database.
                choices:
                    - disable
                    - enable
            nac-quar:
                description:
                    - Configure AntiVirus quarantine settings.
                suboptions:
                    expiry:
                        description:
                            - Duration of quarantine.
                    infected:
                        description:
                            - Enable/Disable quarantining infected hosts to the banned user list.
                        choices:
                            - none
                            - quar-src-ip
                    log:
                        description:
                            - Enable/disable AntiVirus quarantine logging.
                        choices:
                            - enable
                            - disable
            name:
                description:
                    - Profile name.
                required: true
            nntp:
                description:
                    - Configure NNTP AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    options:
                        description:
                            - Enable/disable NNTP AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            pop3:
                description:
                    - Configure POP3 AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    content-disarm:
                        description:
                            - Enable Content Disarm and Reconstruction for this protocol.
                        choices:
                            - disable
                            - enable
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    executables:
                        description:
                            - Treat Windows executable files as viruses for the purpose of blocking or monitoring.
                        choices:
                            - default
                            - virus
                    options:
                        description:
                            - Enable/disable POP3 AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            replacemsg-group:
                description:
                    - Replacement message group customized for this profile. Source system.replacemsg-group.name.
            scan-mode:
                description:
                    - Choose between full scan mode and quick scan mode.
                choices:
                    - quick
                    - full
            smb:
                description:
                    - Configure SMB AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    options:
                        description:
                            - Enable/disable SMB AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
            smtp:
                description:
                    - Configure SMTP AntiVirus options.
                suboptions:
                    archive-block:
                        description:
                            - Select the archive types to block.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    archive-log:
                        description:
                            - Select the archive types to log.
                        choices:
                            - encrypted
                            - corrupted
                            - partiallycorrupted
                            - multipart
                            - nested
                            - mailbomb
                            - fileslimit
                            - timeout
                            - unhandled
                    content-disarm:
                        description:
                            - Enable Content Disarm and Reconstruction for this protocol.
                        choices:
                            - disable
                            - enable
                    emulator:
                        description:
                            - Enable/disable the virus emulator.
                        choices:
                            - enable
                            - disable
                    executables:
                        description:
                            - Treat Windows executable files as viruses for the purpose of blocking or monitoring.
                        choices:
                            - default
                            - virus
                    options:
                        description:
                            - Enable/disable SMTP AntiVirus scanning, monitoring, and quarantine.
                        choices:
                            - scan
                            - avmonitor
                            - quarantine
                    outbreak-prevention:
                        description:
                            - Enable FortiGuard Virus Outbreak Prevention service.
                        choices:
                            - disabled
                            - files
                            - full-archive
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure AntiVirus profiles.
    fortios_antivirus_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      antivirus_profile:
        state: "present"
        analytics-bl-filetype: "3 (source dlp.filepattern.id)"
        analytics-db: "disable"
        analytics-max-upload: "5"
        analytics-wl-filetype: "6 (source dlp.filepattern.id)"
        av-block-log: "enable"
        av-virus-log: "enable"
        comment: "Comment."
        content-disarm:
            cover-page: "disable"
            detect-only: "disable"
            office-embed: "disable"
            office-hylink: "disable"
            office-linked: "disable"
            office-macro: "disable"
            original-file-destination: "fortisandbox"
            pdf-act-form: "disable"
            pdf-act-gotor: "disable"
            pdf-act-java: "disable"
            pdf-act-launch: "disable"
            pdf-act-movie: "disable"
            pdf-act-sound: "disable"
            pdf-embedfile: "disable"
            pdf-hyperlink: "disable"
            pdf-javacode: "disable"
        extended-log: "enable"
        ftgd-analytics: "disable"
        ftp:
            archive-block: "encrypted"
            archive-log: "encrypted"
            emulator: "enable"
            options: "scan"
            outbreak-prevention: "disabled"
        http:
            archive-block: "encrypted"
            archive-log: "encrypted"
            content-disarm: "disable"
            emulator: "enable"
            options: "scan"
            outbreak-prevention: "disabled"
        imap:
            archive-block: "encrypted"
            archive-log: "encrypted"
            content-disarm: "disable"
            emulator: "enable"
            executables: "default"
            options: "scan"
            outbreak-prevention: "disabled"
        inspection-mode: "proxy"
        mapi:
            archive-block: "encrypted"
            archive-log: "encrypted"
            emulator: "enable"
            executables: "default"
            options: "scan"
            outbreak-prevention: "disabled"
        mobile-malware-db: "disable"
        nac-quar:
            expiry: "<your_own_value>"
            infected: "none"
            log: "enable"
        name: "default_name_63"
        nntp:
            archive-block: "encrypted"
            archive-log: "encrypted"
            emulator: "enable"
            options: "scan"
            outbreak-prevention: "disabled"
        pop3:
            archive-block: "encrypted"
            archive-log: "encrypted"
            content-disarm: "disable"
            emulator: "enable"
            executables: "default"
            options: "scan"
            outbreak-prevention: "disabled"
        replacemsg-group: "<your_own_value> (source system.replacemsg-group.name)"
        scan-mode: "quick"
        smb:
            archive-block: "encrypted"
            archive-log: "encrypted"
            emulator: "enable"
            options: "scan"
            outbreak-prevention: "disabled"
        smtp:
            archive-block: "encrypted"
            archive-log: "encrypted"
            content-disarm: "disable"
            emulator: "enable"
            executables: "default"
            options: "scan"
            outbreak-prevention: "disabled"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_antivirus_profile_data(json):
    option_list = ['analytics-bl-filetype', 'analytics-db', 'analytics-max-upload',
                   'analytics-wl-filetype', 'av-block-log', 'av-virus-log',
                   'comment', 'content-disarm', 'extended-log',
                   'ftgd-analytics', 'ftp', 'http',
                   'imap', 'inspection-mode', 'mapi',
                   'mobile-malware-db', 'nac-quar', 'name',
                   'nntp', 'pop3', 'replacemsg-group',
                   'scan-mode', 'smb', 'smtp']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def antivirus_profile(data, fos):
    vdom = data['vdom']
    antivirus_profile_data = data['antivirus_profile']
    filtered_data = filter_antivirus_profile_data(antivirus_profile_data)
    if antivirus_profile_data['state'] == "present":
        return fos.set('antivirus',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif antivirus_profile_data['state'] == "absent":
        return fos.delete('antivirus',
                          'profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_antivirus(data, fos):
    login(data)

    methodlist = ['antivirus_profile']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "antivirus_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "analytics-bl-filetype": {"required": False, "type": "int"},
                "analytics-db": {"required": False, "type": "str",
                                 "choices": ["disable", "enable"]},
                "analytics-max-upload": {"required": False, "type": "int"},
                "analytics-wl-filetype": {"required": False, "type": "int"},
                "av-block-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "av-virus-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "comment": {"required": False, "type": "str"},
                "content-disarm": {"required": False, "type": "dict",
                                   "options": {
                                       "cover-page": {"required": False, "type": "str",
                                                      "choices": ["disable", "enable"]},
                                       "detect-only": {"required": False, "type": "str",
                                                       "choices": ["disable", "enable"]},
                                       "office-embed": {"required": False, "type": "str",
                                                        "choices": ["disable", "enable"]},
                                       "office-hylink": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "office-linked": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "office-macro": {"required": False, "type": "str",
                                                        "choices": ["disable", "enable"]},
                                       "original-file-destination": {"required": False, "type": "str",
                                                                     "choices": ["fortisandbox", "quarantine", "discard"]},
                                       "pdf-act-form": {"required": False, "type": "str",
                                                        "choices": ["disable", "enable"]},
                                       "pdf-act-gotor": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "pdf-act-java": {"required": False, "type": "str",
                                                        "choices": ["disable", "enable"]},
                                       "pdf-act-launch": {"required": False, "type": "str",
                                                          "choices": ["disable", "enable"]},
                                       "pdf-act-movie": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "pdf-act-sound": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "pdf-embedfile": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "pdf-hyperlink": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]},
                                       "pdf-javacode": {"required": False, "type": "str",
                                                        "choices": ["disable", "enable"]}
                                   }},
                "extended-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "ftgd-analytics": {"required": False, "type": "str",
                                   "choices": ["disable", "suspicious", "everything"]},
                "ftp": {"required": False, "type": "dict",
                        "options": {
                            "archive-block": {"required": False, "type": "str",
                                              "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                          "multipart", "nested", "mailbomb",
                                                          "fileslimit", "timeout", "unhandled"]},
                            "archive-log": {"required": False, "type": "str",
                                            "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                        "multipart", "nested", "mailbomb",
                                                        "fileslimit", "timeout", "unhandled"]},
                            "emulator": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                            "options": {"required": False, "type": "str",
                                        "choices": ["scan", "avmonitor", "quarantine"]},
                            "outbreak-prevention": {"required": False, "type": "str",
                                                    "choices": ["disabled", "files", "full-archive"]}
                        }},
                "http": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "content-disarm": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }},
                "imap": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "content-disarm": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "executables": {"required": False, "type": "str",
                                             "choices": ["default", "virus"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }},
                "inspection-mode": {"required": False, "type": "str",
                                    "choices": ["proxy", "flow-based"]},
                "mapi": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "executables": {"required": False, "type": "str",
                                             "choices": ["default", "virus"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }},
                "mobile-malware-db": {"required": False, "type": "str",
                                      "choices": ["disable", "enable"]},
                "nac-quar": {"required": False, "type": "dict",
                             "options": {
                                 "expiry": {"required": False, "type": "str"},
                                 "infected": {"required": False, "type": "str",
                                              "choices": ["none", "quar-src-ip"]},
                                 "log": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]}
                             }},
                "name": {"required": True, "type": "str"},
                "nntp": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }},
                "pop3": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "content-disarm": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "executables": {"required": False, "type": "str",
                                             "choices": ["default", "virus"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }},
                "replacemsg-group": {"required": False, "type": "str"},
                "scan-mode": {"required": False, "type": "str",
                              "choices": ["quick", "full"]},
                "smb": {"required": False, "type": "dict",
                        "options": {
                            "archive-block": {"required": False, "type": "str",
                                              "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                          "multipart", "nested", "mailbomb",
                                                          "fileslimit", "timeout", "unhandled"]},
                            "archive-log": {"required": False, "type": "str",
                                            "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                        "multipart", "nested", "mailbomb",
                                                        "fileslimit", "timeout", "unhandled"]},
                            "emulator": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                            "options": {"required": False, "type": "str",
                                        "choices": ["scan", "avmonitor", "quarantine"]},
                            "outbreak-prevention": {"required": False, "type": "str",
                                                    "choices": ["disabled", "files", "full-archive"]}
                        }},
                "smtp": {"required": False, "type": "dict",
                         "options": {
                             "archive-block": {"required": False, "type": "str",
                                               "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                           "multipart", "nested", "mailbomb",
                                                           "fileslimit", "timeout", "unhandled"]},
                             "archive-log": {"required": False, "type": "str",
                                             "choices": ["encrypted", "corrupted", "partiallycorrupted",
                                                         "multipart", "nested", "mailbomb",
                                                         "fileslimit", "timeout", "unhandled"]},
                             "content-disarm": {"required": False, "type": "str",
                                                "choices": ["disable", "enable"]},
                             "emulator": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                             "executables": {"required": False, "type": "str",
                                             "choices": ["default", "virus"]},
                             "options": {"required": False, "type": "str",
                                         "choices": ["scan", "avmonitor", "quarantine"]},
                             "outbreak-prevention": {"required": False, "type": "str",
                                                     "choices": ["disabled", "files", "full-archive"]}
                         }}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_antivirus(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: docker_config

short_description: Manage docker configs.

version_added: "2.8"

description:
     - Create and remove Docker configs in a Swarm environment. Similar to C(docker config create) and C(docker config rm).
     - Adds to the metadata of new configs 'ansible_key', an encrypted hash representation of the data, which is then used
       in future runs to test if a config has changed. If 'ansible_key' is not present, then a config will not be updated
       unless the C(force) option is set.
     - Updates to configs are performed by removing the config and creating it again.
options:
  data:
    description:
      - The value of the config. Required when state is C(present).
    type: str
  data_is_b64:
    description:
      - If set to C(true), the data is assumed to be Base64 encoded and will be
        decoded before being used.
      - To use binary C(data), it is better to keep it Base64 encoded and let it
        be decoded by this option.
    type: bool
    default: no
  labels:
    description:
      - "A map of key:value meta data, where both the I(key) and I(value) are expected to be a string."
      - If new meta data is provided, or existing meta data is modified, the config will be updated by removing it and creating it again.
    type: dict
  force:
    description:
      - Use with state C(present) to always remove and recreate an existing config.
      - If I(true), an existing config will be replaced, even if it has not been changed.
    type: bool
    default: no
  name:
    description:
      - The name of the config.
    type: str
    required: yes
  state:
    description:
      - Set to C(present), if the config should exist, and C(absent), if it should not.
    type: str
    default: present
    choices:
      - absent
      - present

extends_documentation_fragment:
  - docker
  - docker.docker_py_2_documentation

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 2.6.0"
  - "Docker API >= 1.30"

author:
  - Chris Houseknecht (@chouseknecht)
  - John Hu (@ushuz)
'''

EXAMPLES = '''

- name: Create config foo (from a file on the control machine)
  docker_config:
    name: foo
    # If the file is JSON or binary, Ansible might modify it (because
    # it is first decoded and later re-encoded). Base64-encoding the
    # file directly after reading it prevents this to happen.
    data: "{{ lookup('file', '/path/to/config/file') | b64encode }}"
    data_is_b64: true
    state: present

- name: Change the config data
  docker_config:
    name: foo
    data: Goodnight everyone!
    labels:
      bar: baz
      one: '1'
    state: present

- name: Add a new label
  docker_config:
    name: foo
    data: Goodnight everyone!
    labels:
      bar: baz
      one: '1'
      # Adding a new label will cause a remove/create of the config
      two: '2'
    state: present

- name: No change
  docker_config:
    name: foo
    data: Goodnight everyone!
    labels:
      bar: baz
      one: '1'
      # Even though 'two' is missing, there is no change to the existing config
    state: present

- name: Update an existing label
  docker_config:
    name: foo
    data: Goodnight everyone!
    labels:
      bar: monkey   # Changing a label will cause a remove/create of the config
      one: '1'
    state: present

- name: Force the (re-)creation of the config
  docker_config:
    name: foo
    data: Goodnight everyone!
    force: yes
    state: present

- name: Remove config foo
  docker_config:
    name: foo
    state: absent
'''

RETURN = '''
config_id:
  description:
    - The ID assigned by Docker to the config object.
  returned: success and C(state == "present")
  type: str
  sample: 'hzehrmyjigmcp2gb6nlhmjqcv'
'''

import base64
import hashlib

try:
    from docker.errors import APIError
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

from ansible.module_utils.docker.common import AnsibleDockerClient, DockerBaseClass, compare_generic
from ansible.module_utils._text import to_native, to_bytes


class ConfigManager(DockerBaseClass):

    def __init__(self, client, results):

        super(ConfigManager, self).__init__()

        self.client = client
        self.results = results
        self.check_mode = self.client.check_mode

        parameters = self.client.module.params
        self.name = parameters.get('name')
        self.state = parameters.get('state')
        self.data = parameters.get('data')
        if self.data is not None:
            if parameters.get('data_is_b64'):
                self.data = base64.b64decode(self.data)
            else:
                self.data = to_bytes(self.data)
        self.labels = parameters.get('labels')
        self.force = parameters.get('force')
        self.data_key = None

    def __call__(self):
        if self.state == 'present':
            self.data_key = hashlib.sha224(self.data).hexdigest()
            self.present()
        elif self.state == 'absent':
            self.absent()

    def get_config(self):
        ''' Find an existing config. '''
        try:
            configs = self.client.configs(filters={'name': self.name})
        except APIError as exc:
            self.client.fail("Error accessing config %s: %s" % (self.name, to_native(exc)))

        for config in configs:
            if config['Spec']['Name'] == self.name:
                return config
        return None

    def create_config(self):
        ''' Create a new config '''
        config_id = None
        # We can't see the data after creation, so adding a label we can use for idempotency check
        labels = {
            'ansible_key': self.data_key
        }
        if self.labels:
            labels.update(self.labels)

        try:
            if not self.check_mode:
                config_id = self.client.create_config(self.name, self.data, labels=labels)
        except APIError as exc:
            self.client.fail("Error creating config: %s" % to_native(exc))

        if isinstance(config_id, dict):
            config_id = config_id['ID']

        return config_id

    def present(self):
        ''' Handles state == 'present', creating or updating the config '''
        config = self.get_config()
        if config:
            self.results['config_id'] = config['ID']
            data_changed = False
            attrs = config.get('Spec', {})
            if attrs.get('Labels', {}).get('ansible_key'):
                if attrs['Labels']['ansible_key'] != self.data_key:
                    data_changed = True
            labels_changed = not compare_generic(self.labels, attrs.get('Labels'), 'allow_more_present', 'dict')
            if data_changed or labels_changed or self.force:
                # if something changed or force, delete and re-create the config
                self.absent()
                config_id = self.create_config()
                self.results['changed'] = True
                self.results['config_id'] = config_id
        else:
            self.results['changed'] = True
            self.results['config_id'] = self.create_config()

    def absent(self):
        ''' Handles state == 'absent', removing the config '''
        config = self.get_config()
        if config:
            try:
                if not self.check_mode:
                    self.client.remove_config(config['ID'])
            except APIError as exc:
                self.client.fail("Error removing config %s: %s" % (self.name, to_native(exc)))
            self.results['changed'] = True


def main():
    argument_spec = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', default='present', choices=['absent', 'present']),
        data=dict(type='str'),
        data_is_b64=dict(type='bool', default=False),
        labels=dict(type='dict'),
        force=dict(type='bool', default=False)
    )

    required_if = [
        ('state', 'present', ['data'])
    ]

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=required_if,
        min_docker_version='2.6.0',
        min_docker_api_version='1.30',
    )

    results = dict(
        changed=False,
    )

    ConfigManager(client, results)()
    client.module.exit_json(**results)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_monitor_snmp_dca import Parameters
    from library.modules.bigip_monitor_snmp_dca import ModuleManager
    from library.modules.bigip_monitor_snmp_dca import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_monitor_snmp_dca import Parameters
    from ansible.modules.network.f5.bigip_monitor_snmp_dca import ModuleManager
    from ansible.modules.network.f5.bigip_monitor_snmp_dca import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            agent_type='UCD',
            community='public',
            cpu_coefficient='1.5',
            cpu_threshold='80',
            parent='/Common/snmp_dca',
            disk_coefficient='2.0',
            disk_threshold='90',
            interval=10,
            memory_coefficient='1.0',
            memory_threshold='70',
            time_until_up=0,
            timeout=30,
            version='v1'
        )

        p = Parameters(params=args)
        assert p.agent_type == 'UCD'
        assert p.community == 'public'
        assert p.cpu_coefficient == 1.5
        assert p.cpu_threshold == 80
        assert p.parent == '/Common/snmp_dca'
        assert p.disk_coefficient == 2.0
        assert p.disk_threshold == 90
        assert p.interval == 10
        assert p.memory_coefficient == 1.0
        assert p.memory_threshold == 70
        assert p.time_until_up == 0
        assert p.timeout == 30
        assert p.version == 'v1'

    def test_api_parameters(self):
        args = dict(
            agentType='UCD',
            community='public',
            cpuCoefficient='1.5',
            cpuThreshold='80',
            defaultsFrom='/Common/snmp_dca',
            diskCoefficient='2.0',
            diskThreshold='90',
            interval=10,
            memoryCoefficient='1.0',
            memoryThreshold='70',
            timeUntilUp=0,
            timeout=30,
            apiRawValues={
                "userDefined asdasd": "{ foo }",
                "userDefined bar": "tim rupp",
                "user-defined baz-": "nia",
                "userDefined userDefined": "23234"
            },
            version='v1'
        )

        p = Parameters(params=args)
        assert p.agent_type == 'UCD'
        assert p.community == 'public'
        assert p.cpu_coefficient == 1.5
        assert p.cpu_threshold == 80
        assert p.parent == '/Common/snmp_dca'
        assert p.disk_coefficient == 2.0
        assert p.disk_threshold == 90
        assert p.interval == 10
        assert p.memory_coefficient == 1.0
        assert p.memory_threshold == 70
        assert p.time_until_up == 0
        assert p.timeout == 30
        assert p.version == 'v1'


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

    def test_create(self, *args):
        set_module_args(dict(
            name='foo',
            agent_type='UCD',
            community='public',
            cpu_coefficient='1.5',
            cpu_threshold='80',
            parent='/Common/snmp_dca',
            disk_coefficient='2.0',
            disk_threshold='90',
            memory_coefficient='1.0',
            memory_threshold='70',
            version='v1',
            interval=20,
            timeout=30,
            time_until_up=60,
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )

        # Override methods in the specific type of manager
        mm = ModuleManager(module=module)
        mm.exists = Mock(side_effect=[False, True])
        mm.create_on_device = Mock(return_value=True)

        results = mm.exec_module()

        assert results['changed'] is True



# (c) 2018-2019, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

""" unit tests for Ansible module: na_ontap_aggregate """

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_aggregate \
    import NetAppOntapAggregate as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None, parm2=None):
        ''' save arguments '''
        self.type = kind
        self.parm1 = parm1
        self.parm2 = parm2
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'aggregate':
            xml = self.build_aggregate_info(self.parm1, self.parm2)
        elif self.type == 'aggregate_fail':
            raise netapp_utils.zapi.NaApiError(code='TEST', message="This exception is from the unit test")
        self.xml_out = xml
        return xml

    @staticmethod
    def build_aggregate_info(vserver, aggregate):
        ''' build xml data for aggregatte and vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        data = {'num-records': 2,
                'attributes-list':
                    {'aggr-attributes':
                        {'aggregate-name': aggregate,
                         'aggr-raid-attributes':
                             {'state': 'offline'
                              }
                         },
                     },
                'vserver-info':
                    {'vserver-name': vserver
                     }
                }
        xml.translate_struct(data)
        print(xml.to_string())
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection('aggregate', '12', 'name')
        # whether to use a mock or a simulator
        self.onbox = False

    def set_default_args(self):
        if self.onbox:
            hostname = '10.193.74.78'
            username = 'admin'
            password = 'netapp1!'
            name = 'name'
        else:
            hostname = 'hostname'
            username = 'username'
            password = 'password'
            name = 'name'
        return dict({
            'hostname': hostname,
            'username': username,
            'password': password,
            'name': name
        })

    def call_command(self, module_args):
        ''' utility function to call apply '''
        module_args.update(self.set_default_args())
        set_module_args(module_args)
        my_obj = my_module()
        my_obj.asup_log_for_cserver = Mock(return_value=None)
        if not self.onbox:
            # mock the connection
            my_obj.server = MockONTAPConnection('aggregate', '12', 'test_name')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        return exc.value.args[0]['changed']

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_is_mirrored(self):
        module_args = {
            'disk_count': '2',
            'is_mirrored': 'true',
        }
        changed = self.call_command(module_args)
        assert not changed

    def test_disks_list(self):
        module_args = {
            'disk_count': '2',
            'disks': ['1', '2'],
        }
        changed = self.call_command(module_args)
        assert not changed

    def test_mirror_disks(self):
        module_args = {
            'disk_count': '2',
            'disks': ['1', '2'],
            'mirror_disks': ['3', '4']
        }
        changed = self.call_command(module_args)
        assert not changed

    def test_spare_pool(self):
        module_args = {
            'disk_count': '2',
            'spare_pool': 'Pool1'
        }
        changed = self.call_command(module_args)
        assert not changed

    def test_rename(self):
        module_args = {
            'from_name': 'test_name2'
        }
        changed = self.call_command(module_args)
        assert not changed

    def test_if_all_methods_catch_exception(self):
        module_args = {}
        module_args.update(self.set_default_args())
        module_args.update({'service_state': 'online'})
        module_args.update({'unmount_volumes': 'True'})
        module_args.update({'from_name': 'test_name2'})
        set_module_args(module_args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('aggregate_fail')
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.aggr_get_iter(module_args.get('name'))
        assert '' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.aggregate_online()
        assert 'Error changing the state of aggregate' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.aggregate_offline()
        assert 'Error changing the state of aggregate' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.create_aggr()
        assert 'Error provisioning aggregate' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.delete_aggr()
        assert 'Error removing aggregate' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.rename_aggregate()
            assert 'Error renaming aggregate' in exc.value.args[0]['msg']
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.asup_log_for_cserver = Mock(return_value=None)
            my_obj.apply()
            assert 'Error renaming: aggregate test_name2 does not exist' in exc.value.args[0]['msg']



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_nfs \
    import NetAppONTAPNFS as nfs_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None, job_error=None):
        ''' save arguments '''
        self.kind = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'nfs':
            xml = self.build_nfs_info(self.params)
        self.xml_out = xml
        if self.kind == 'nfs_status':
            xml = self.build_nfs_status_info(self.params)
        return xml

    @staticmethod
    def build_nfs_info(nfs_details):
        ''' build xml data for volume-attributes '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            "attributes-list": {
                "nfs-info": {
                    "auth-sys-extended-groups": "false",
                    "cached-cred-harvest-timeout": "86400000",
                    "cached-cred-negative-ttl": "7200000",
                    "cached-cred-positive-ttl": "86400000",
                    "cached-transient-err-ttl": "30000",
                    "chown-mode": "use_export_policy",
                    "enable-ejukebox": "true",
                    "extended-groups-limit": "32",
                    "file-session-io-grouping-count": "5000",
                    "file-session-io-grouping-duration": "120",
                    "ignore-nt-acl-for-root": "false",
                    "is-checksum-enabled-for-replay-cache": "true",
                    "is-mount-rootonly-enabled": "true",
                    "is-netgroup-dns-domain-search": "true",
                    "is-nfs-access-enabled": "false",
                    "is-nfs-rootonly-enabled": "false",
                    "is-nfsv2-enabled": "false",
                    "is-nfsv3-64bit-identifiers-enabled": "false",
                    "is-nfsv3-connection-drop-enabled": "true",
                    "is-nfsv3-enabled": "true",
                    "is-nfsv3-fsid-change-enabled": "true",
                    "is-nfsv4-fsid-change-enabled": "true",
                    "is-nfsv4-numeric-ids-enabled": "true",
                    "is-nfsv40-acl-enabled": "false",
                    "is-nfsv40-enabled": "true",
                    "is-nfsv40-migration-enabled": "false",
                    "is-nfsv40-read-delegation-enabled": "false",
                    "is-nfsv40-referrals-enabled": "false",
                    "is-nfsv40-req-open-confirm-enabled": "false",
                    "is-nfsv40-write-delegation-enabled": "false",
                    "is-nfsv41-acl-enabled": "false",
                    "is-nfsv41-acl-preserve-enabled": "true",
                    "is-nfsv41-enabled": "true",
                    "is-nfsv41-migration-enabled": "false",
                    "is-nfsv41-pnfs-enabled": "true",
                    "is-nfsv41-read-delegation-enabled": "false",
                    "is-nfsv41-referrals-enabled": "false",
                    "is-nfsv41-state-protection-enabled": "true",
                    "is-nfsv41-write-delegation-enabled": "false",
                    "is-qtree-export-enabled": "false",
                    "is-rquota-enabled": "false",
                    "is-tcp-enabled": "false",
                    "is-udp-enabled": "false",
                    "is-v3-ms-dos-client-enabled": "false",
                    "is-validate-qtree-export-enabled": "true",
                    "is-vstorage-enabled": "false",
                    "map-unknown-uid-to-default-windows-user": "true",
                    "mountd-port": "635",
                    "name-service-lookup-protocol": "udp",
                    "netgroup-trust-any-ns-switch-no-match": "false",
                    "nfsv4-acl-max-aces": "400",
                    "nfsv4-grace-seconds": "45",
                    "nfsv4-id-domain": "defaultv4iddomain.com",
                    "nfsv4-lease-seconds": "30",
                    "nfsv41-implementation-id-domain": "netapp.com",
                    "nfsv41-implementation-id-name": "NetApp Release Kalyaniblack__9.4.0",
                    "nfsv41-implementation-id-time": "1541070767",
                    "nfsv4x-session-num-slots": "180",
                    "nfsv4x-session-slot-reply-cache-size": "640",
                    "nlm-port": "4045",
                    "nsm-port": "4046",
                    "ntacl-display-permissive-perms": "false",
                    "ntfs-unix-security-ops": "use_export_policy",
                    "permitted-enc-types": {
                        "string": ["des", "des3", "aes_128", "aes_256"]
                    },
                    "rpcsec-ctx-high": "0",
                    "rpcsec-ctx-idle": "0",
                    "rquotad-port": "4049",
                    "showmount": "true",
                    "showmount-timestamp": "1548372452",
                    "skip-root-owner-write-perm-check": "false",
                    "tcp-max-xfer-size": "1048576",
                    "udp-max-xfer-size": "32768",
                    "v3-search-unconverted-filename": "false",
                    "v4-inherited-acl-preserve": "false",
                    "vserver": "ansible"
                }
            },
            "num-records": "1"
        }
        xml.translate_struct(attributes)
        return xml

    @staticmethod
    def build_nfs_status_info(nfs_status_details):
        ''' build xml data for volume-attributes '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'is-enabled': "true"
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.mock_nfs_group = {
            'vserver': 'nfs_vserver',
        }

    def mock_args(self):
        return {
            'vserver': self.mock_nfs_group['vserver'],
            'hostname': 'test',
            'username': 'test_user',
            'password': 'test_pass!',
            'https': 'False'
        }

    def get_nfs_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_volume object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_volume object
        """
        nfsy_obj = nfs_module()
        nfsy_obj.asup_log_for_cserver = Mock(return_value=None)
        nfsy_obj.cluster = Mock()
        nfsy_obj.cluster.invoke_successfully = Mock()
        if kind is None:
            nfsy_obj.server = MockONTAPConnection()
        else:
            nfsy_obj.server = MockONTAPConnection(kind=kind, data=self.mock_nfs_group)
        return nfsy_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            nfs_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_get_nonexistent_nfs(self):
        ''' Test if get_nfs_service returns None for non-existent nfs '''
        set_module_args(self.mock_args())
        result = self.get_nfs_mock_object().get_nfs_service()
        assert result is None

    def test_get_existing_nfs(self):
        ''' Test if get_policy_group returns details for existing nfs '''
        set_module_args(self.mock_args())
        result = self.get_nfs_mock_object('nfs').get_nfs_service()
        assert result['is_nfsv3_enabled']

    def test_get_nonexistent_nfs_status(self):
        ''' Test if get__nfs_status returns None for non-existent nfs '''
        set_module_args(self.mock_args())
        result = self.get_nfs_mock_object().get_nfs_status()
        assert result is None

    def test_get_existing_nfs_status(self):
        ''' Test if get__nfs_status returns details for nfs '''
        set_module_args(self.mock_args())
        result = self.get_nfs_mock_object('nfs_status').get_nfs_status()
        assert result

    def test_modify_nfs(self):
        ''' Test if modify_nfs runs for existing nfs '''
        data = self.mock_args()
        data['nfsv3'] = 'enabled'
        data['nfsv3_fsid_change'] = 'enabled'
        data['nfsv4'] = 'enabled'
        data['nfsv41'] = 'enabled'
        data['vstorage_state'] = 'enabled'
        data['tcp'] = 'enabled'
        data['udp'] = 'enabled'
        data['nfsv4_id_domain'] = 'nfsv4_id_domain'
        data['nfsv40_acl'] = 'enabled'
        data['nfsv40_read_delegation'] = 'enabled'
        data['nfsv40_write_delegation'] = 'enabled'
        data['nfsv41_acl'] = 'enabled'
        data['nfsv41_read_delegation'] = 'enabled'
        data['nfsv41_write_delegation'] = 'enabled'
        data['showmount'] = 'enabled'
        data['tcp_max_xfer_size'] = '1048576'
        set_module_args(data)
        self.get_nfs_mock_object('nfs_status').modify_nfs()

    def test_successfully_modify_nfs(self):
        ''' Test modify nfs successful for modifying tcp max xfer size. '''
        data = self.mock_args()
        data['tcp_max_xfer_size'] = '8192'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_nfs_mock_object('nfs').apply()
        assert exc.value.args[0]['changed']

    def test_modify_nfs_idempotency(self):
        ''' Test modify nfs idempotency '''
        data = self.mock_args()
        data['tcp_max_xfer_size'] = '1048576'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_nfs_mock_object('nfs').apply()
        assert not exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_nfs.NetAppONTAPNFS.delete_nfs')
    def test_successfully_delete_nfs(self, delete_nfs):
        ''' Test successfully delete nfs '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        obj = self.get_nfs_mock_object('nfs')
        with pytest.raises(AnsibleExitJson) as exc:
            obj.apply()
        assert exc.value.args[0]['changed']
        delete_nfs.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_nfs.NetAppONTAPNFS.get_nfs_service')
    def test_successfully_enable_nfs(self, get_nfs_service):
        ''' Test successfully enable nfs on non-existent nfs '''
        data = self.mock_args()
        data['state'] = 'present'
        set_module_args(data)
        get_nfs_service.side_effect = [
            None,
            {}
        ]
        obj = self.get_nfs_mock_object('nfs')
        with pytest.raises(AnsibleExitJson) as exc:
            obj.apply()
        assert exc.value.args[0]['changed']



# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import traceback

REQUESTS_IMP_ERR = None
try:
    import requests
    HAS_REQUESTS = True
except ImportError:
    REQUESTS_IMP_ERR = traceback.format_exc()
    HAS_REQUESTS = False

PYVMOMI_IMP_ERR = None
try:
    from pyVim import connect
    from pyVmomi import vim, vmodl
    HAS_PYVMOMI = True
except ImportError:
    PYVMOMI_IMP_ERR = traceback.format_exc()
    HAS_PYVMOMI = False

VCLOUD_IMP_ERR = None
try:
    from vmware.vapi.lib.connect import get_requests_connector
    from vmware.vapi.security.session import create_session_security_context
    from vmware.vapi.security.user_password import create_user_password_security_context
    from com.vmware.cis_client import Session
    from com.vmware.vapi.std_client import DynamicID
    HAS_VCLOUD = True
except ImportError:
    VCLOUD_IMP_ERR = traceback.format_exc()
    HAS_VCLOUD = False

VSPHERE_IMP_ERR = None
try:
    from vmware.vapi.stdlib.client.factories import StubConfigurationFactory
    HAS_VSPHERE = True
except ImportError:
    VSPHERE_IMP_ERR = traceback.format_exc()
    HAS_VSPHERE = False

from ansible.module_utils._text import to_native
from ansible.module_utils.basic import env_fallback, missing_required_lib


class VmwareRestClient(object):
    def __init__(self, module):
        """
        Constructor

        """
        self.module = module
        self.params = module.params
        self.check_required_library()
        self.connect = self.connect_to_rest()

    def check_required_library(self):
        """
        Check required libraries

        """
        if not HAS_REQUESTS:
            self.module.fail_json(msg=missing_required_lib('requests'),
                                  exception=REQUESTS_IMP_ERR)
        if not HAS_PYVMOMI:
            self.module.fail_json(msg=missing_required_lib('PyVmomi'),
                                  exception=PYVMOMI_IMP_ERR)
        if not HAS_VSPHERE:
            self.module.fail_json(
                msg=missing_required_lib('vSphere Automation SDK',
                                         url='https://code.vmware.com/web/sdk/65/vsphere-automation-python'),
                exception=VSPHERE_IMP_ERR)
        if not HAS_VCLOUD:
            self.module.fail_json(
                msg=missing_required_lib('vCloud Suite SDK',
                                         url='https://code.vmware.com/web/sdk/60/vcloudsuite-python'),
                exception=VCLOUD_IMP_ERR)

    def connect_to_rest(self):
        """
        Connect to server using username and password

        """
        session = requests.Session()
        session.verify = self.params.get('validate_certs')

        username = self.params.get('username', None)
        password = self.params.get('password', None)
        protocol = self.params.get('protocol', 'https')
        hostname = self.params.get('hostname')

        if not all([self.params.get('hostname', None), username, password]):
            self.module.fail_json(msg="Missing one of the following : hostname, username, password."
                                      " Please read the documentation for more information.")

        vcenter_url = "%s://%s/api" % (protocol, hostname)

        # Get request connector
        connector = get_requests_connector(session=session, url=vcenter_url)
        # Create standard Configuration
        stub_config = StubConfigurationFactory.new_std_configuration(connector)
        # Use username and password in the security context to authenticate
        security_context = create_user_password_security_context(username, password)
        # Login
        stub_config.connector.set_security_context(security_context)
        # Create the stub for the session service and login by creating a session.
        session_svc = Session(stub_config)
        session_id = None
        try:
            session_id = session_svc.create()
        except OSError as os_err:
            self.module.fail_json(msg="Failed to login to %s: %s" % (hostname,
                                                                     to_native(os_err)))

        if session_id is None:
            self.module.fail_json(msg="Failed to create session using provided credentials."
                                      " Please check hostname, username and password.")
        # After successful authentication, store the session identifier in the security
        # context of the stub and use that for all subsequent remote requests
        session_security_context = create_session_security_context(session_id)
        stub_config.connector.set_security_context(session_security_context)

        if stub_config is None:
            self.module.fail_json(msg="Failed to login to %s" % hostname)
        return stub_config

    @staticmethod
    def vmware_client_argument_spec():
        return dict(
            hostname=dict(type='str',
                          fallback=(env_fallback, ['VMWARE_HOST'])),
            username=dict(type='str',
                          fallback=(env_fallback, ['VMWARE_USER']),
                          aliases=['user', 'admin']),
            password=dict(type='str',
                          fallback=(env_fallback, ['VMWARE_PASSWORD']),
                          aliases=['pass', 'pwd'],
                          no_log=True),
            protocol=dict(type='str',
                          default='https',
                          choices=['https', 'http']),
            validate_certs=dict(type='bool',
                                fallback=(env_fallback, ['VMWARE_VALIDATE_CERTS']),
                                default=True),
        )

    def get_tags_for_object(self, tag_service, tag_assoc_svc, dobj):
        """
        Return list of tag objects associated with an object
        Args:
            dobj: Dynamic object
            tag_service: Tag service object
            tag_assoc_svc: Tag Association object
        Returns: List of tag objects associated with the given object
        """
        tag_ids = tag_assoc_svc.list_attached_tags(dobj)
        tags = []
        for tag_id in tag_ids:
            tags.append(tag_service.get(tag_id))
        return tags

    def get_vm_tags(self, tag_service, tag_association_svc, vm_mid=None):
        """
        Return list of tag name associated with virtual machine
        Args:
            tag_service:  Tag service object
            tag_association_svc: Tag association object
            vm_mid: Dynamic object for virtual machine

        Returns: List of tag names associated with the given virtual machine

        """
        tags = []
        if vm_mid is None:
            return tags
        dynamic_managed_object = DynamicID(type='VirtualMachine', id=vm_mid)

        temp_tags_model = self.get_tags_for_object(tag_service, tag_association_svc, dynamic_managed_object)
        for t in temp_tags_model:
            tags.append(t.name)
        return tags

    @staticmethod
    def search_svc_object_by_name(service, svc_obj_name=None):
        """
        Return service object by name
        Args:
            service: Service object
            svc_obj_name: Name of service object to find

        Returns: Service object if found else None

        """
        if not svc_obj_name:
            return None

        for svc_object in service.list():
            svc_obj = service.get(svc_object)
            if svc_obj.name == svc_obj_name:
                return svc_obj
        return None
